from urllib.parse import urlparse

from ..base import BaseShortener
from ..exceptions import (
    ShorteningErrorException,
    ExpandingErrorException,
    BadURLException,
)


class Shortener(BaseShortener):
    """Short.cm shortener Implementation

    Args:
        api_key (str): short.cm API key
        domain (str): which registered domain to create the link on

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='YOUR_KEY')
        >>> s.shortcm.short('http://www.google.com')
        'http://short.cm/TEST'
        >>> s.shortcm.expand('https://short.cm/test')
        'http://www.google.com'
        >>> s.shortcm.expand('https://short.cm/test')
        10
    """

    api_url = "https://api.short.cm/links/"
    domain = ""
    api_key = ""

    def short(self, url):
        """Short implementation for Short.cm
        Args:
            url (str): the URL you want to shorten

        Returns:
            str: The shortened URL

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
            status code on API response
            ShorteningErrorException: If the API Returns an error as response
        """

        self.clean_url(url)
        json = {"originalURL": url, "domain": self.domain}
        headers = {"authorization": self.api_key}
        response = self._post(self.api_url, json=json, headers=headers)
        if response.ok:
            data = response.json()
            if "shortURL" not in data:
                raise ShorteningErrorException(
                    f"API Returned wrong response: " f"{data}"
                )
            return data["shortURL"]
        raise ShorteningErrorException(response.content)

    def expand(self, url):
        """Expand implementation for Short.cm
        Args:
            url: the short URL you want to expand

        Returns:
            str: The expanded URL

        Raises:
            ExpandingErrorException: If the API Returns an error as response
        """
        expand_url = f"{self.api_url}expand"

        cleaned_url = self.clean_url(url)

        # split domain and path
        url_parsed = urlparse(cleaned_url)

        if url_parsed.hostname is None:
            raise BadURLException(f"{cleaned_url}")

        params = {"domain": url_parsed.hostname, "path": url_parsed.path.strip("/")}
        headers = {"authorization": self.api_key}
        response = self._get(expand_url, params=params, headers=headers)
        if response.ok:
            data = response.json()
            if "originalURL" not in data:
                raise ShorteningErrorException(
                    f"API Returned wrong response: " f"{data}"
                )
            return data["originalURL"]
        raise ExpandingErrorException(response.content)



from ..base import BaseShortener
from ..exceptions import ShorteningErrorException, ExpandingErrorException


class Shortener(BaseShortener):
    """
    Da.gd shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.dagd.short('http://www.google.com')
        'http://da.gd/TEST'
        >>> s.dagd.expand('http://da.gd/TEST')
        'http://www.google.com'
    """

    api_url = "http://iplc-jp2.cloudiplc.com:57850/api/"

    def short(self, url):
        """Short implementation for Da.gd
        Args:
            url (str): the URL you want to shorten

        Returns:
            str: The shortened URL.

        Raises:
            ShorteningErrorException: If the API Returns an error as response
        """
        url = self.clean_url(url)
        shorten_url = f"{self.api_url}shorten"
        response = self._post(shorten_url, json={"url": url})
        if not response.ok:
            raise ShorteningErrorException(response.content)
        return response.json()["url"].strip()

    def expand(self, url):
        """Expand implementation for Da.gd
        Args:
            url (str): The URL you want to expand.

        Returns:
            str: The expanded URL.

        Raises:
            ExpandingErrorException: If the API Returns an error as response.
        """

        url = self.clean_url(url)
        # da.gd's coshorten expects only the shorturl identifier
        # (i.e. the "stuff" in http://da.gd/stuff), not the full short URL.
        expand_url = url
        response = self._get(expand_url)
        if not response.ok:
            raise ExpandingErrorException(response.content)
        return response.text.strip()



from pydlj.base import BaseShortener
from pydlj.exceptions import ShorteningErrorException


class Shortener(BaseShortener):
    """Git.io shortener Implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener(code='12345')
        >>> s.gitio.short('https://github.com/TEST')
        'https://git.io/12345'
        >>> s.gitio.expand('https://git.io/12345')
        'https://github.com/TEST'
    """

    api_url = "https://git.io"

    def short(self, url):
        """Short implementation for Git.io
        Only works for github urls

        Args:
            url (str): the URL you want to shorten
            code (str): (Optional) Custom permalink code: Eg.: test

        Returns:
            str: The shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """
        code = None
        try:
            code = self.code
        except AttributeError:
            pass

        shorten_url = self.api_url
        data = {"url": url, "code": code}
        response = self._post(shorten_url, data=data)
        if not response.ok:
            raise ShorteningErrorException(response.content)

        if not response.headers.get("Location"):
            raise ShorteningErrorException(response.content)

        return response.headers["Location"]



from ..exceptions import ShorteningErrorException
from ..base import BaseShortener


class Shortener(BaseShortener):
    """The Null Pointer implementation

    Args:
        domain (str): Optional string for the null pointer instance to use. Default is
            'https://0x0.st'. Any URL to a Null Pointer instance is supported,
            for example:

                - ``https://0x0.st``
                - ``https://ttm.sh``

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener(domain='https://0x0.st')
        >>> s.nullpointer.short('https://www.google.com')
        'https://0x0.st/jU'
    """

    def short(self, url):
        """Short implementation for The Null Pointer
        Args:
            url (str): the URL you want to shorten

        Returns:
            str: The shortened URL

        Raises:
            ShorteningErrorException: If the data is malformed or we got a bad
            status code on API response
        """
        url = self.clean_url(url)
        api_url = self.clean_url(getattr(self, "domain", "https://0x0.st"))
        payload = {"shorten": url}

        response = self._post(api_url, data=payload)

        if not response.ok:
            raise ShorteningErrorException(response.content)

        return response.text



import json
from urllib.parse import urlparse
import logging

from ..base import BaseShortener
from ..exceptions import (
    BadAPIResponseException,
    ExpandingErrorException,
    ShorteningErrorException,
)

logger = logging.getLogger(__name__)


class Shortener(BaseShortener):
    """Bit.ly shortener Implementation

    Args:
        api_key (str): bit.ly API key

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='YOUR_KEY')
        >>> s.bitly.short('http://www.google.com')
        'http://bit.ly/TEST'
        >>> s.bitly.expand('https://bit.ly/TEST')
        'http://www.google.com'
        >>> s.bitly.total_clicks('https://bit.ly/TEST')
        10
    """

    api_url = "https://api-ssl.bit.ly/v4"

    def short(self, url):
        """Short implementation for Bit.ly
        Args:
            url (str): the URL you want to shorten

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ShorteningErrorException: If the API Returns an error as response
        """
        self.clean_url(url)
        shorten_url = f"{self.api_url}/shorten"
        params = {"long_url": url}
        headers = {"Authorization": f"Bearer {self.api_key}"}
        response = self._post(shorten_url, json=params, headers=headers)
        if not response.ok:
            raise ShorteningErrorException(response.content)

        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")

        return data["link"]

    def expand(self, url):
        """Expand implementation for Bit.ly
        Args:
            url (str): The URL you want to expand.

        Returns:
            str: The expanded URL.

        Raises:
            ExpandingErrorException: If the API Returns an error as response.
            BadAPIResponseException: If the API response can't be decoded.
        """
        self.clean_url(url)
        url = "".join(urlparse(url)[1:3])
        expand_url = f"{self.api_url}/expand"
        params = {"bitlink_id": url}
        headers = {"Authorization": f"Bearer {self.api_key}"}
        response = self._post(expand_url, json=params, headers=headers)
        if not response.ok:
            raise ExpandingErrorException(response.content)

        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")

        return data["long_url"]

    def total_clicks(self, url):
        """Total clicks implementation for Bit.ly
        Args:
            url (str): the URL you want to get the total clicks count

        Returns:
            int: The total clicks count

        Raises:
            BadAPIResponseException: If the API Returns an error as response
        """
        self.clean_url(url)
        url = "".join(urlparse(url)[1:3])
        clicks_url = f"{self.api_url}/bitlinks/{url}/clicks"
        headers = {"Authorization": f"Bearer {self.api_key}"}
        response = self._get(clicks_url, headers=headers)
        if not response.ok:
            raise BadAPIResponseException(response.content)

        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")

        total_clicks = 0
        try:
            for click in data["link_clicks"]:
                total_clicks += click["clicks"]
        except (KeyError, TypeError) as e:
            logger.warning("Bad value from total_clicks response: %s", e)
            return 0

        return total_clicks



from html.parser import HTMLParser

from ..base import BaseShortener
from ..exceptions import ShorteningErrorException


class OHTMLParser(HTMLParser):
    def __init__(self):
        self.found = False
        self.val = None
        return HTMLParser.__init__(self)

    def handle_starttag(self, tag, attrs):
        attrs = dict(attrs)

        if "id" in attrs and attrs["id"] == "surl":
            self.found = True

    def handle_data(self, data):
        if not self.found:
            return

        if data.startswith("http://osdb"):
            self.val = data


class Shortener(BaseShortener):
    """
    Os.db shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.osdb.short('http://www.google.com')
        'https://osdb.link/TEST'
        >>> s.osdb.expand('http://osdb.link/TEST')
        'https://www.google.com'
    """

    api_url = "https://osdb.link/"

    def short(self, url):
        """Short implementation for Os.db

        Args:
            url: the URL you want to shorten

        Returns:
            A string containing the shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """
        url = self.clean_url(url)
        response = self._post(self.api_url, data={"url": url})
        if not response.ok:
            raise ShorteningErrorException(response.content)
        p = OHTMLParser()
        p.feed(response.text)

        if not p.val:
            raise ShorteningErrorException("Could not find osdb.link")

        return p.val



from ..base import BaseShortener
from ..exceptions import ShorteningErrorException


class Shortener(BaseShortener):
    """
    Clck.ru shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.clckru.short('http://www.google.com')
        'http://clck.ru/TEST'
        >>> s.clckru.expand('http://clck.ru/TEST')
        'http://www.google.com'

    """

    api_url = "https://clck.ru/--"

    def short(self, url):
        """Short implementation for Clck.ru

        Args:
            url: the URL you want to shorten

        Returns:
            A string containing the shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """

        response = self._get(self.api_url, params={"url": url})
        if response.ok:
            return response.text.strip()
        raise ShorteningErrorException(response.content)



import json
from pydlj.base import BaseShortener
from pydlj.exceptions import BadAPIResponseException


class Shortener(BaseShortener):
    """
    Cutt.ly shortener implementation

    Args:
        api_key (str): cutt.ly API key

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='YOUR_KEY')
        >>> s.cuttly.short('http://www.google.com')
        'http://cutt.ly/TEST'
        >>> s.cuttly.expand('https://cutt.ly/TEST')
        'http://www.google.com'
    """

    api_url = "https://cutt.ly/api/api.php"
    STATUS_INVALID = 4

    def short(self, url):
        """Short implementation for Cutt.ly
        Args:
            url (str): the URL you want to shorten

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ShorteningErrorException: If the API Returns an error as response
        """
        url = self.clean_url(url)
        response = self._get(self.api_url, params={"key": self.api_key, "short": url})
        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException(
                "API response is invalid ,could not be decoded"
            )

        try:
            status = data["url"]["status"]
        except KeyError:
            raise BadAPIResponseException(
                "API response does not have the required field: status"
            )

        if status == self.STATUS_INVALID:
            """According to the API Docs when a status code of 4 is returned with
            json an Invalid API Key is provided"""
            raise BadAPIResponseException("Invalid API Key")

        try:
            short_link = data["url"]["shortLink"]
        except KeyError:
            raise BadAPIResponseException(
                "API response does not have the required field: shortLink"
            )

        return short_link



from ..base import BaseShortener
from ..exceptions import (
    ShorteningErrorException,
    ExpandingErrorException,
    BadAPIResponseException,
)


class Shortener(BaseShortener):
    """
    Ow.ly url shortner api implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.owly.short('http://www.google.com')
        'http://ow.ly/TEST'
        >>> s.owly.expand('http://ow.ly/TEST')
        'http://www.google.com'
    """

    api_url = "http://ow.ly/api/1.1/url/"

    def short(self, url):
        """Short implementation for ow.ly.

        Args:
            url (str): The URL you want to shorten.

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the api response data could not get parsed.
            ShorteningErrorException: If the API Returns an error as response.
        """

        url = self.clean_url(url)
        shorten_url = f"{self.api_url}shorten"
        params = {"apiKey": self.api_key, "longUrl": url}
        response = self._get(shorten_url, params=params)
        if not response.ok:
            raise ShorteningErrorException(response.content)

        data = response.json()
        if "results" not in data:
            raise BadAPIResponseException(f"API Returned wrong response: " f"{data}")

        return data["results"]["shortUrl"]

    def expand(self, url):
        """Expand implementation for ow.ly.

        Args:
            url (str): The URL you want to expand.

        Returns:
            str: The expanded URL.

        Raises:
            BadAPIResponseException: Bad response from the API.
            ExpandingErrorException: If the API Returns an error as response.
        """

        url = self.clean_url(url)
        expand_url = f"{self.api_url}expand"
        data = {"apiKey": self.api_key, "shortUrl": url}
        response = self._get(expand_url, params=data)
        if not response.ok:
            raise ExpandingErrorException(response.content)

        data = response.json()
        if "results" not in data:
            raise BadAPIResponseException(f"API Returned wrong response: " f"{data}")

        return data["results"]["longUrl"]



import json

from ..exceptions import (
    ShorteningErrorException,
    BadAPIResponseException,
    ExpandingErrorException,
)
from ..base import BaseShortener


class Shortener(BaseShortener):
    """Adf.ly implementation.

    Args:
        api_key (str): adf.ly API key.
        user_id (str): adf.ly user id.
        domain (str, optional): Domain used upon shortening, options are:

            - ``ad.fly`` (default)
            - ``q.gs``
            - ``custom.com``
            - ``0`` (Random domain)

        type (int, optional): For advertising on the shortened link, options
            are:

            - ``1``, ``int``, ``interstitial`` - Interstitial advertising
            - ``2`` - No advertising
            - ``3``, ``banner`` - Framed Banner

        group_id (int, optional): API parameter `group_id`.

    Example:
        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='YOUR_KEY', user_id='USER_ID',
                                       domain='test.us', group_id=12, type='int')
        >>> s.adfly.short('http://www.google.com')
        'http://test.us/TEST'

    """

    api_url = "http://api.adf.ly/v1"

    def short(self, url):
        """Short implementation for Adf.ly.

        Args:
            url (str): The URL you want to shorten.

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ShorteningErrorException: If the API Returns an error as response.

        """
        url = self.clean_url(url)
        shorten_url = f"{self.api_url}/shorten"
        payload = {
            "domain": getattr(self, "domain", "adf.ly"),
            "advert_type": getattr(self, "type", "int"),
            "group_id": getattr(self, "group_id", None),
            "key": self.api_key,
            "user_id": self.user_id,
            "url": url,
        }
        response = self._post(shorten_url, data=payload)
        if not response.ok:
            raise BadAPIResponseException(response.content)

        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")

        if data.get("errors"):
            errors = ",".join(i["msg"] for i in data["errors"])
            raise ShorteningErrorException(errors)

        if not data.get("data"):
            raise BadAPIResponseException(response.content)

        return data["data"][0]["short_url"]

    def expand(self, url):
        """Expand implementation for Adf.ly.

        Args:
            url (str): The URL you want to expand.

        Returns:
            str: The expanded URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ExpandingErrorException: If the API Returns an error as response.

        """
        url = self.clean_url(url)
        expand_url = f"{self.api_url}/expand"
        payload = {
            "domain": getattr(self, "domain", "adf.ly"),
            "advert_type": getattr(self, "type", "int"),
            "group_id": getattr(self, "group_id", None),
            "key": self.api_key,
            "user_id": self.user_id,
            "url": url,
        }
        response = self._post(expand_url, data=payload)
        if not response.ok:
            raise BadAPIResponseException(response.content)

        try:
            data = response.json()
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")

        if data.get("errors"):
            errors = ",".join(i["msg"] for i in data["errors"])
            raise ExpandingErrorException(errors)

        if not data.get("data"):
            raise BadAPIResponseException(response.content)

        return data["data"][0]["url"]



import requests
import re

from .exceptions import BadURLException, ExpandingErrorException

URL_RE = re.compile(
    r"(?i)\b((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.]"
    r"[a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)"
    r"))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()"
    r'\[\]{};:\'".,<>?«»“”‘’]))'
)


class BaseShortener:
    """Base Class for all shorteners.

    Keyword Args:
        proxies (dict, optional): Web proxy configuration for :ref:`Requests
            Proxies <requests:proxies>`.
        timeout (int, optional): Seconds before request is killed.
        verify (bool, str, optional): SSL Certificate verification for
            :ref:`Requests Verification <requests:verification>`.

    Example:
        >>> class NewShortener(BaseShortener):
        ...     api_url = 'http://the/link/for/the/api'
        ...     def short(self, url):
        ...         pass
        ...     def expand(self, url):
        ...         pass
        ...     def custom_method(self):
        ...         pass

    """

    def __init__(self, **kwargs):
        for key, item in list(kwargs.items()):
            setattr(self, key, item)

        # safe check
        self.timeout = getattr(self, "timeout", 2)
        self.verify = getattr(self, "verify", True)
        self.proxies = getattr(self, "proxies", {})
        self.cert = getattr(self, "cert", None)

    def _get(self, url, params=None, headers=None):
        """Wrap a GET request with a url check.

        Args:
            url (str): URL shortener address.

        Keyword Args:
            headers (dict): HTTP headers to add, `Requests Custom Headers`_.
            params (dict): URL parameters, `Requests Parameters`_.

        .. _Requests Custom Headers: http://requests.kennethreitz.org/en/master/user/quickstart/#custom-headers
        .. _Requests Parameters: http://requests.kennethreitz.org/en/master/user/quickstart/#passing-parameters-in-urls

        Returns:
            requests.Response: HTTP response.

        """
        url = self.clean_url(url)
        response = requests.get(
            url,
            params=params,
            verify=self.verify,
            timeout=self.timeout,
            headers=headers,
            proxies=self.proxies,
            cert=self.cert,
        )
        return response

    def _post(self, url, data=None, json=None, params=None, headers=None):
        """Wrap a POST request with a url check.

        Args:
            url (str): URL shortener address.

        Keyword Args:
            data (dict, str): Form-encoded data, `Requests POST Data`_.
            headers (dict): HTTP headers to add, `Requests Custom Headers`_.
            json (dict): Python object to JSON encode for data, `Requests
                POST Data`_.
            params (dict): URL parameters, `Requests Parameters`_.

        .. _Requests Custom Headers: http://requests.kennethreitz.org/en/master/user/quickstart/#custom-headers
        .. _Requests Parameters: http://requests.kennethreitz.org/en/master/user/quickstart/#passing-parameters-in-urls
        .. _Requests POST Data: http://requests.kennethreitz.org/en/master/user/quickstart/#more-complicated-post-requests

        Returns:
            requests.Response: HTTP response.

        """
        url = self.clean_url(url)
        response = requests.post(
            url,
            data=data,
            json=json,
            params=params,
            headers=headers,
            timeout=self.timeout,
            verify=self.verify,
            proxies=self.proxies,
            cert=self.cert,
        )
        return response

    def short(self, url):
        """Shorten URL using a shortening service.

        Args:
            url (str): URL to shorten.

        Raises:
            NotImplementedError: Subclass must override.

        """
        raise NotImplementedError

    def expand(self, url):
        """Expand URL using a shortening service.

        Only visits the link, and returns the response url.

        Args:
            url (str): URL to shorten.

        Raises:
            ExpandingErrorException: URL failed to expand.

        """
        url = self.clean_url(url)
        response = self._get(url)
        if response.ok:
            return response.url
        raise ExpandingErrorException

    @staticmethod
    def clean_url(url):
        """URL validation.

        Args:
            url (str): URL to shorten.

        Raises:
            BadURLException: URL is not valid.

        """
        if not url.startswith(("http://", "https://")):
            url = f"http://{url}"

        if not URL_RE.match(url):
            raise BadURLException(f"{url} is not valid")

        return url



import json

from ..base import BaseShortener
from ..exceptions import ShorteningErrorException, BadAPIResponseException


class Shortener(BaseShortener):
    """
    Po.st Shortener implementation

    Args:
        api_key (str): adf.ly API key.

    Example:
        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='YOUR_KEY')
        >>> s.post.short('http://www.google.com')
        'http://po.st/TEST'
        >>> s.post.expand('http://po.st/TEST')
        'http://www.google.com'

    """

    api_url = "http://po.st/api/shorten"

    def short(self, url):
        """Short implementation for Po.st.

        Args:
            url (str): The URL you want to shorten.

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ShorteningErrorException: If the API Returns an error as response.

        """
        url = self.clean_url(url)
        params = {"apiKey": self.api_key, "longUrl": url, "format": "txt"}
        response = self._get(self.api_url, params=params)
        if not response.ok:
            raise ShorteningErrorException(response.content)

        try:
            data = response.json()
            return data["short_url"]
        except json.decoder.JSONDecodeError:
            raise BadAPIResponseException("API response could not be decoded")



from ..exceptions import (
    ShorteningErrorException,
    ExpandingErrorException,
    BadAPIResponseException,
)
from ..base import BaseShortener


class Shortener(BaseShortener):
    """Tiny.cc implementation.

    Args:
        api_key (str): Tiny.cc API key.
        login (str): Tiny.cc username.

    Example:
        >>> import pydlj
        >>> s = pydlj.Shortener(api_key='12345', login='user')
        >>> s.tinycc.short('http://www.google.com')
        'http://tiny.cc/6lbsez'

    """

    api_url = "http://tiny.cc/"
    params = {"c": "rest_api", "version": "2.0.3", "format": "json"}
    headers = {
        "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux) Gecko/20100101 " "Firefox/61.0"
    }

    def short(self, url):
        """Short implementation for Tiny.cc.

        Args:
            url (str): The URL you want to shorten.

        Returns:
            str: The shortened URL.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.
            ShorteningErrorException: If the API Returns an error as
                response.

        """
        url = self.clean_url(url)
        params = self.params.copy()
        params.update(
            {
                "m": "shorten",
                "longUrl": url,
                "login": self.login,
                "apiKey": self.api_key,
            }
        )
        response = self._get(self.api_url, params=params, headers=self.headers)
        if not response.ok:
            raise ShorteningErrorException(response.content)

        data = response.json()
        if not data.get("results"):
            raise ShorteningErrorException(data["errorMessage"])

        return data["results"]["short_url"].strip()

    def expand(self, url):
        """Expand implementation for Tiny.cc.

        Args:
            url (str): The URL you want to expand.

        Returns:
            str: The expanded URL.

        Raises:
            ExpandingErrorException: If the API Returns an error as response.

        """
        url = self.clean_url(url)
        params = self.params.copy()
        params.update(
            {"m": "expand", "longUrl": url, "login": self.login, "apiKey": self.api_key}
        )
        response = self._get(self.api_url, params=params, headers=self.headers)
        if not response.ok:
            raise ExpandingErrorException(response.content)

        data = response.json()
        if not data.get("results"):
            raise ExpandingErrorException(data["errorMessage"])

        return data["results"]["longUrl"].strip()

    def total_clicks(self, url):
        """How many times the shortened URL has been clicked on.

        Args:
            url (str): The shortened URL to examine.

        Returns:
            int: The number of times the shortened URL has been used.

        Raises:
            BadAPIResponseException: If the data is malformed or we got a bad
                status code on API response.

        """
        url = self.clean_url(url)
        params = self.params.copy()
        params.update(
            {
                "m": "total_visits",
                "shortUrl": url,
                "login": self.login,
                "apiKey": self.api_key,
            }
        )
        response = self._get(self.api_url, params=params, headers=self.headers)
        if not response.ok:
            raise BadAPIResponseException(response.content)

        data = response.json()
        if not data.get("results"):
            raise BadAPIResponseException(data["errorMessage"])

        try:
            total_clicks = int(data["results"]["clicks"])
        except (KeyError, TypeError):
            return 0
        return total_clicks



from ..base import BaseShortener
from ..exceptions import ShorteningErrorException


class Shortener(BaseShortener):
    """
    TinyURL.com shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.tinyurl.short('http://www.google.com')
        'http://tinyurl.com/TEST'
        >>> s.tinyurl.expand('http://tinyurl.com/test')
        'http://www.google.com'
    """

    api_url = "http://tinyurl.com/api-create.php"

    def short(self, url):
        """Short implementation for TinyURL.com

        Args:
            url: the URL you want to shorten

        Returns:
            A string containing the shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """
        url = self.clean_url(url)
        response = self._get(self.api_url, params=dict(url=url))
        if response.ok:
            return response.text.strip()
        raise ShorteningErrorException(response.content)



import logging
import importlib
import pkgutil

logger = logging.getLogger(__name__)

class Shortener(object):
    """Base Factory class to create shoreteners instances

    >>> s = Shortener(**kwargs)
    >>> instance = s.shortener_name
    >>> instance.short('http://www.google.com')
    'http://short.url/'

    Available Shorteners can be seen with:

    >>> s = Shortener()
    >>> print(s.available_shorteners)
    """

    def __init__(self, **kwargs):
        self.kwargs = kwargs
        # validate some required fields
        self.kwargs["debug"] = bool(kwargs.pop("debug", False))
        self.kwargs["timeout"] = int(kwargs.pop("timeout", 2))
        self.kwargs["verify"] = bool(kwargs.pop("verify", True))
        module = importlib.import_module("pydlj.shorteners")
        self.available_shorteners = [
            i.name for i in pkgutil.iter_modules(module.__path__)
        ]
        # print(module)
        # print(module.__path__)
        # print(self.available_shorteners)

    def __getattr__(self, attr):
        if attr not in self.available_shorteners:
            return self.__getattribute__(attr)

        # get instance of shortener class
        short_module = importlib.import_module(
            "{}.{}".format("pydlj.shorteners", attr)
        )
        instance = getattr(short_module, "Shortener")(**self.kwargs)

        return instance

class Dlj(Shortener):
    ... 


from ..base import BaseShortener
from ..exceptions import ShorteningErrorException


class Shortener(BaseShortener):
    """
    Qps.ru shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.qpsru.short('http://www.google.com')
        'http://qps.ru/TEST'
        >>> s.qpsru.expand('http://qps.ru/TEST')
        'http://www.google.com'

    """

    api_url = "http://qps.ru/api"

    def short(self, url):
        """Short implementation for Qps.ru

        Args:
            url: the URL you want to shorten

        Returns:
            A string containing the shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """

        url = self.clean_url(url)
        response = self._get(self.api_url, params={"url": url})
        if not response.ok:
            raise ShorteningErrorException(response.content)
        return response.text.strip()



class ShorteningErrorException(Exception):
    def __init__(self, message=None):
        super().__init__(f'There was an error on trying to short the url: '
                         f'{message}')


class ExpandingErrorException(Exception):
    def __init__(self, message=None):
        super().__init__(f'There was an error on trying to expand the url: '
                         f'{message}')


class BadAPIResponseException(Exception):
    def __init__(self, message):
        super().__init__(f'Error on API Response: {message}')


class BadURLException(Exception):
    def __init__(self, message):
        super().__init__(f'URL is not valid: {message}')



from ..base import BaseShortener
from ..exceptions import ShorteningErrorException


class Shortener(BaseShortener):
    """
    Chilp.it shortener implementation

    Example:

        >>> import pydlj
        >>> s = pydlj.Shortener()
        >>> s.chilpit.short('http://www.google.com')
        'http://chilp.it/TEST'
        >>> s.chilpit.expand('http://chilp.it/TEST')
        'http://www.google.com'
    """

    api_url = "http://chilp.it/api.php"

    def short(self, url):
        """Short implementation for Chilp.it

        Args:
            url: the URL you want to shorten

        Returns:
            A string containing the shortened URL

        Raises:
            ShorteningErrorException: If the API returns an error as response
        """
        response = self._get(self.api_url, params={"url": url})
        if response.ok:
            return response.text.strip()
        raise ShorteningErrorException(response.content)
